@@ -19,7 +19,7 @@
 ###########################################################################
 
 hergm <- function(formula, 
-                  max_number = NULL,
+                  max_number = 2,
                   hierarchical = TRUE,
                   parametric = FALSE,
                   parameterization = "standard",
@@ -73,13 +73,19 @@ hergm <- function(formula,
   # if (sum(network[,] == 1) == 0) stop("\nNetwork is extreme: terminating...\n\n") # Simplistic check
   control$drop <- FALSE
   model <- ergm_model(formula, network, drop=control$drop, expanded=TRUE)
+  Clist <- ergm.Cprepare(network, model)
+  for (i in 1:Clist$nterms) 
+    {
+    if (model$terms[[i]]$name %in% c("edges_i", "arcs_i", "arcs_j", "ctriple_ijk", "ttriple_ijk")) method <- "bayes"
+    }
+  if ((simulate == FALSE) && (is.null(indicator) == FALSE)) method <- "bayes" 
   if (is.null(sample_size)) 
     {
     if (method == "ml") sample_size <- 2500
     else sample_size = 1e+5 
     }
+  if ((method == "bayes") && (simulate == FALSE) && (sample_size < 100)) sample_size <- 100
   MCMCsamplesize <- sample_size
-  Clist <- ergm.Cprepare(network, model)
   ## Commenting old line and putting changed line underneath 
   ## ergm.design changed in ergm-master / changing for compatibility 
   #Clist.miss <- ergm.design(network, model, verbose=FALSE)
@@ -97,7 +103,6 @@ hergm <- function(formula,
   MCMCparams$stats <- matrix(0,ncol=Clist$nstats,nrow=s)
   MCMCparams$target.stats <- Clist$target.stats
   object <- hergm.mcmc(parameterization, method, sample_size_multiplier_blocks, original.formula, max_number, initialize, initialization_method, network, model, hyper_prior=hierarchical, parametric, MHproposal, MCMCparams, verbose, scaling, alpha_shape, alpha_rate, alpha, eta_mean_mean, eta_mean_sd, eta_precision_shape, eta_precision_rate, eta_mean, eta_sd, mean_between, eta, indicator, parallel, simulate, seeds, mh.scale, variational, temperature, predictions, perturb, estimate_parameters, n_em_step_max, max_iter, initial_estimate = initial_estimate, NR_step_len = NR_step_len, NR_step_len_multiplier = NR_step_len_multiplier, NR_max_iter = NR_max_iter)
-  if (is.null(max_number)) max_number <- 1
   if ((max_number >= 10) && (relabel == 1)) relabel <- 2 
   if ((simulate == FALSE) && (method == "bayes")) 
     {
